import config
from base import *
try:
    import psyco
    psyco.full()
except:
    print "Psyco not found.  Running in slow mode"
    
class PygameEngine(Engine):
    def __init__(self, width, height):
        Engine.__init__(self,width,height)
        import pygame
        import pygame.font
        import pygame.time

        self.clock = pygame.time.Clock()
        self.tile_size = 50
        self.phevo_size = 50
        self.bug_size = 45

        self.pygame = pygame
        try:
            flags = pygame.SRCALPHA+pygame.HWSURFACE
            self.pygame.init()
            self.win = pygame.display.set_mode((self.width, self.height), pygame.RESIZABLE, 32)
            self.pygame.display.set_caption(config.title)
            
            self.phevo_surfaces = range(config.players)
            
            for player_id in range(config.players):
                self.phevo_surfaces[player_id] = pygame.Surface((self.phevo_size * 8, self.phevo_size * 8), flags=flags)#//.convert()
                self.phevo_surfaces[player_id].fill((48,48,48))

            self.back_surface = pygame.Surface((self.width, self.height), flags=flags)#.convert()
            #self.back_surface.fill((0,0,0))

            self.background_surface = pygame.image.load(config.background).convert()
            
            self.win.blit(self.background_surface,(0,0))
            
            self.innershape_surface = pygame.image.load(config.innershapes).convert_alpha()
            self.outershape_surface = pygame.image.load(config.outershapes).convert_alpha()

            self.deathicon_surface = pygame.image.load(config.deathicon).convert_alpha()
            self.sickicon_surface = pygame.image.load(config.sickicon).convert_alpha()

            self.syringe_surfaces = range(config.players)
            self.syringeover_surfaces = range(config.players)
            self.cursors = range(config.players)
            for player_id in range(config.players):
                self.syringe_surfaces[player_id] = pygame.Surface((self.bug_size * 5,self.bug_size), flags=flags)
                self.syringe_surfaces[player_id].set_alpha(255)
                self.syringe_surfaces[player_id].fill((192,192,192))

                self.syringeover_surfaces[player_id] = pygame.Surface((self.bug_size * 5,self.bug_size), flags=flags)
                self.syringeover_surfaces[player_id].set_alpha(48)
                self.syringeover_surfaces[player_id].fill((0,128,128))
                
                self.cursors[player_id] = Cursor(self,player_id)

            self.syringe_font = pygame.font.Font(pygame.font.get_default_font(),12)
            self.syringe_font.set_bold(True)

            self.finished = False
            self.clear()
        except Exception, e:
            self.stop()
            raise e

    def __del__(self):
        self.pygame.quit()

    def clear(self):
        self.back_surface.fill((0,0,0))

    def stop(self):
        self.pygame.quit()

    def on_phevopop_populated(self,pop,phevo):
        self.render_phevo(phevo)

    def on_phevo_infected(self,phevo):
        self.render_phevo(phevo)

    def on_phevo_cured(self,phevo):
        self.render_phevo(phevo)

    def on_phevo_died(self,phevo):
        self.render_phevo(phevo)

    def on_phevo_promoted(self,phevo):
        self.render_phevo(phevo)
        
    def on_bug_mutated(self,bug):
        self.render_syringe(bug.syringe)
        #self.render_bug(bug)

    def on_syringe_filled(self,syringe,bug):
        self.render_syringe(syringe)
        """self.syringe_surfaces[bug.syringe.player.id].fill((128,128,128))
        for index,next_bug in enumerate(syringe.bugs):
            right = (index) * self.bug_size
            self.render_bug(next_bug,right)
            #self.debug("PLAYER ID %d" % (next_bug.syringe.player.id))
        self.draw()
        """
        """
        if not syringe.player.AI:
            self.syringe_surfaces[syringe.player.id].fill((128,128,128))
            for index,bug in enumerate(syringe.bugs):
                right = index * self.bug_size
                self.render_bug(bug,right)
            self.debug("syringe %d filled animated" % syringe.id)
        """
        
    
    def render_syringe(self,syringe):
        self.syringe_surfaces[syringe.player.id].fill((128,128,128))
        for index,next_bug in enumerate(syringe.bugs):
            right = (index) * self.bug_size
            self.render_bug(next_bug,right)
    
    def on_syringe_sprayed(self,syringe,population,bug):
        self.debug("PLAYER ID %d" % (syringe.player.id))
        if not syringe.player == population.player:
            self.render_syringe(syringe)
            #self.syringe_surfaces[bug.syringe.player.id].fill((128,128,128))
            #for frame in range(15):
            #self.syringe_surfaces[syringe.player.id].fill((128,128,128))
                #render the sprayed bug
            #    right = -3 * frame
            #    self.render_bug(bug,right)
                #and move the remaining
            #for index,next_bug in enumerate(syringe.bugs):
            #    right = (index + 1) * self.bug_size
            #    self.render_bug(next_bug,right)
                    #self.debug("PLAYER ID %d" % (next_bug.syringe.player.id))
            #self.draw()
            #self.debug("syringe %d spray animated" % syringe.id)
            
    def on_player_defeated(self,player):
        self.debug("Player %d has been defeated. GAME OVER" % (player.id + 1))
        self.finished = True
        self.notify()
    
    def on_cursor_updated(self,cursor):
        self.updated = True
    
    def render_bug(self,bug,right=0,anim=False):
        #self.syringe_surface.fill((128,128,128))
        #self.debug(bug.id)
        #self.syringe_surfaces[bug.syringe.player.id].fill((128,128,128))
        
        if (bug.mask >> 2) & 3 == 3:
            outershape_x = bug.pheno[0] * self.tile_size
        else:
            outershape_x = self.tile_size * 4
            
        if (bug.mask) & 3 == 3:
            outershape_y = bug.pheno[1] * self.tile_size
        else:
            outershape_y = self.tile_size * 5
            
        self.syringe_surfaces[bug.syringe.player.id].blit(self.outershape_surface,(right,0),(outershape_x,outershape_y,self.tile_size,self.tile_size))
        
        self.updated = True
        

    def render_phevo(self,phevo):
        outershape_x = phevo.pheno[0] * self.tile_size
        outeralpha = 255

        """
        if phevo.infected:
            outershape_y = self.tile_size * 4
            innershape_y = self.tile_size * 4
        elif phevo.dead:
            outershape_y = self.tile_size * 5
            innershape_y = self.tile_size * 5
        else:
            outershape_y = phevo.pheno[0][1] * self.tile_size
            innershape_y = phevo.pheno[1][1] * self.tile_size
        """
        
        icon = None
        if phevo.infected:
            icon = self.sickicon_surface
        elif phevo.dead:
            icon = self.deathicon_surface

        outershape_y = phevo.pheno[1] * self.tile_size
        
        if phevo.population:
            player_id = phevo.population.player.id
            self.phevo_surfaces[player_id].fill((255,255,255),((phevo.index % 8) * self.phevo_size, phevo.index/8 * self.phevo_size,self.tile_size,self.tile_size))
            if not (phevo.promoted or phevo.dead):
            #if not (phevo.promoted):
                self.phevo_surfaces[player_id].blit(self.outershape_surface,((phevo.index % 8) * self.phevo_size, phevo.index/8 * self.phevo_size),(outershape_x,outershape_y,self.tile_size,self.tile_size))
                if icon:
                    self.phevo_surfaces[player_id].blit(icon,((phevo.index % 8) * self.phevo_size, phevo.index/8 * self.phevo_size))
            
        self.updated = True
        self.debug("rendering %s" % phevo)

    def tick(self):
        self.notify("ticked")
        self.clock.tick(config.framerate)

    def poll_controller(self):    
        for event in self.pygame.event.get():
            if event.type == self.pygame.QUIT:
                self.finished = True
                self.notify()
            elif event.type == self.pygame.KEYDOWN:
                if event.key in (self.pygame.K_q, self.pygame.K_ESCAPE):
                    self.finished = True
                    self.notify()
                elif event.key == self.pygame.K_w:
                    self.notify("cursor_move_requested",0,0)
                    self.debug("Key down!")
                elif event.key == self.pygame.K_d:
                    self.notify("cursor_move_requested",0,1)
                elif event.key == self.pygame.K_s:
                    self.notify("cursor_move_requested",0,2)
                elif event.key == self.pygame.K_a:
                    self.notify("cursor_move_requested",0,3)
                elif event.key == self.pygame.K_z:
                    self.notify("promotion_requested",0)
                elif event.key == self.pygame.K_x:
                    self.notify("spray_requested",0)
                elif event.key == self.pygame.K_i:
                    self.notify("cursor_move_requested",1,0)
                elif event.key == self.pygame.K_l:
                    self.notify("cursor_move_requested",1,1)
                elif event.key == self.pygame.K_k:
                    self.notify("cursor_move_requested",1,2)
                elif event.key == self.pygame.K_j:
                    self.notify("cursor_move_requested",1,3)
                elif event.key == self.pygame.K_n:
                    self.notify("promotion_requested",1)
                elif event.key == self.pygame.K_m:
                    self.notify("spray_requested",1)

    def draw(self):
        if self.updated:
            rects = []
            rects.append(self.win.blit(self.background_surface,(0,0)))
            rects.append(self.win.blit(self.phevo_surfaces[0],(0,0)))
            rects.append(self.win.blit(self.phevo_surfaces[1],(self.phevo_surfaces[0].get_width() + config.margin,0)))
            rects.append(self.win.blit(self.syringe_surfaces[0],(0,self.phevo_surfaces[0].get_height() + config.margin)))
            rects.append(self.win.blit(self.syringe_surfaces[1],(self.phevo_surfaces[0].get_width() + config.margin,self.phevo_surfaces[0].get_height() + config.margin)))
            rects.append(self.win.blit(self.syringeover_surfaces[0],(0,self.phevo_surfaces[0].get_height() + config.margin)))
            rects.append(self.win.blit(self.syringeover_surfaces[1],(self.phevo_surfaces[0].get_width() + config.margin,self.phevo_surfaces[1].get_height() + config.margin)))
            self.pygame.draw.rect(self.win,self.pygame.color.Color("#ff0000"),self.pygame.rect.Rect(self.cursors[0].location[0]*self.phevo_size,self.cursors[0].location[1]*self.phevo_size,self.tile_size,self.tile_size),2)
            self.pygame.draw.rect(self.win,self.pygame.color.Color("#ff0000"),self.pygame.rect.Rect(self.phevo_surfaces[0].get_width() + config.margin + self.cursors[1].location[0]*self.phevo_size,self.cursors[1].location[1]*self.phevo_size,self.tile_size,self.tile_size),2)
            self.pygame.display.update(rects)
            
            self.clear()
            self.updated = False

    def on_pygameengine_ticked(self,engine):
        self.poll_controller()
        self.draw()

#TODO: This needs fixing.  This is properly a model object, not a view object
class Cursor(Agent):
    def __init__(self,engine,id):
        super(Cursor,self).__init__()
        self.engine = engine
        self.id = id
        self.location = [0,0]
        
    def on_pygameengine_cursor_move_requested(self,engine,id,direction):
        if id == self.id:
            if direction % 2 == 0:
                self.location[1] += direction - 1 #converts 0 and 2 into -1 and 1
                if self.location[1] < 0:
                    self.location[1] = 0
                if self.location[1] >= config.grid_height:
                    self.location[1] = config.grid_height - 1
            else:
                self.location[0] += (direction - 2) * -1 #converts 3 and 1 into -1 and 1
                if self.location[0] < 0:
                    self.location[0] = 0
                if self.location[0] >= config.grid_width:
                    self.location[0] = config.grid_width - 1
            self.debug(self.location)
            self.notify("updated")
            
    def on_pygameengine_promotion_requested(self,engine,player_id):
        if self.id == player_id:
            index = self.location[0] + self.location[1] * config.grid_width
            self.notify("promotion_located",player_id,index)

    def on_pygameengine_spray_requested(self,engine,player_id):
        if self.id == player_id:
            index = self.location[0] + self.location[1] * config.grid_width
            self.notify("spray_located",player_id,index)
        